"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_codegen_visitor_plugin_common_1 = require("graphql-codegen-visitor-plugin-common");
var autoBind = require("auto-bind");
var graphql_1 = require("graphql");
var R_MOD = /module\:\s*"([^"]+)"/; // matches: module: "..."
var R_NAME = /name\:\s*"([^"]+)"/; // matches: name: "..."
function R_DEF(directive) {
    return new RegExp("\\s+\\@" + directive + "\\([^)]+\\)", 'gm');
}
var ApolloAngularVisitor = /** @class */ (function (_super) {
    __extends(ApolloAngularVisitor, _super);
    function ApolloAngularVisitor(fragments, _allOperations, rawConfig) {
        var _this = _super.call(this, fragments, rawConfig, {}) || this;
        _this._allOperations = _allOperations;
        autoBind(_this);
        return _this;
    }
    ApolloAngularVisitor.prototype.getImports = function () {
        var _this = this;
        var baseImports = _super.prototype.getImports.call(this);
        var imports = ["import { Injectable } from '@angular/core';", "import * as Apollo from 'apollo-angular';"];
        var defs = {};
        this._allOperations
            .filter(function (op) { return _this._operationHasDirective(op, 'NgModule'); })
            .forEach(function (op) {
            var def = _this._extractNgModule(op);
            // by setting key as link we easily get rid of duplicated imports
            // every path should be relative to the output file
            defs[def.link] = {
                path: def.path,
                module: def.module
            };
        });
        Object.keys(defs).forEach(function (key) {
            var def = defs[key];
            // Every Angular Module that I've seen in my entire life use named exports
            imports.push("import { " + def.module + " } from '" + def.path + "';");
        });
        return [baseImports].concat(imports).join('\n');
    };
    ApolloAngularVisitor.prototype._extractNgModule = function (operation) {
        var _a = graphql_1.print(operation).match(R_MOD), link = _a[1];
        var _b = link.split('#'), path = _b[0], module = _b[1];
        return {
            path: path,
            module: module,
            link: link
        };
    };
    ApolloAngularVisitor.prototype._operationHasDirective = function (operation, directive) {
        return graphql_1.print(operation).includes("@" + directive);
    };
    ApolloAngularVisitor.prototype._removeDirective = function (document, directive) {
        if (this._operationHasDirective(document, directive)) {
            return graphql_1.parse(graphql_1.print(document).replace(R_DEF(directive), '')).definitions[0];
        }
        return graphql_1.parse(graphql_1.print(document)).definitions[0];
    };
    ApolloAngularVisitor.prototype._removeDirectives = function (document, directives) {
        var _this = this;
        return directives.reduce(function (doc, directive) { return _this._removeDirective(doc, directive); }, document);
    };
    ApolloAngularVisitor.prototype._extractDirective = function (operation, directive) {
        var directives = graphql_1.print(operation).match(R_DEF(directive));
        if (directives.length > 1) {
            throw new Error("The " + directive + " directive used multiple times in '" + operation.name + "' operation");
        }
        return directives[0];
    };
    ApolloAngularVisitor.prototype._prepareDocument = function (documentStr) {
        return graphql_1.print(this._removeDirectives(graphql_1.parse(documentStr).definitions[0], ['NgModule', 'namedClient']));
    };
    ApolloAngularVisitor.prototype._namedClient = function (operation) {
        if (!this._operationHasDirective(operation, 'namedClient')) {
            return '';
        }
        var name = this._extractNamedClient(operation);
        return "client = '" + name + "';";
    };
    // tries to find namedClient directive and extract {name}
    ApolloAngularVisitor.prototype._extractNamedClient = function (operation) {
        var _a = this._extractDirective(operation, 'namedClient').match(R_NAME), name = _a[1];
        return name;
    };
    ApolloAngularVisitor.prototype._providedIn = function (operation) {
        if (this._operationHasDirective(operation, 'NgModule')) {
            return this._extractNgModule(operation).module;
        }
        return "'root'";
    };
    ApolloAngularVisitor.prototype.buildOperation = function (node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        var content = "\n  @Injectable({\n    providedIn: " + this._providedIn(node) + "\n  })\n  export class " + this.convertName(node.name.value) + "GQL extends Apollo." + operationType + "<" + operationResultType + ", " + operationVariablesTypes + "> {\n    document = " + documentVariableName + ";\n    " + this._namedClient(node) + "\n  }";
        return content;
    };
    return ApolloAngularVisitor;
}(graphql_codegen_visitor_plugin_common_1.ClientSideBaseVisitor));
exports.ApolloAngularVisitor = ApolloAngularVisitor;
//# sourceMappingURL=visitor.js.map